local ARP = LibStub("AceAddon-3.0"):NewAddon("ARP", "AceTimer-3.0")

-- Version Data
ARP.VERSION = 'r75'
ARP.REVISION = '75'
ARP.versionList = {}

-- Keybindings Text
BINDING_HEADER_ARP		= "Angry Raid Points"
BINDING_NAME_ARP_TOGGLE = "Toggle Window"

----------------------)
---Global Variables---)
----------------------)

-- Configurable
local rankMember = 4          -- lowest guild rank of Members
local rankMemberException = 2 -- exception to the above (in case you have an "officer alt" rank, if none, just set to -1)
local rankTrial = 5           -- guild rank of Trials

local minBidMMS = 10          -- minimum bid for members, mainspec
local minBidMOS = 1           -- minimum bid for members, offspec
local minBidTMS = 10          -- minimum bid for trials, mainspec
local minBidTOS = 1           -- minimum bid for trials, offspec

-- Any items with these ilvls will be suffixed with (H) in the loot log
-- (Used to differentiate Heroic loot from Normal loot)
local heroicLoot = { 502, 509, 535, }	
-- Any items with these ilvls will be suffixed with (ilvl) in the loot log
-- (Used to differentiate Elite and Thunderforged loot from their regular counterparts)
local ilvlLoot = { 503, 516, 528, 541, }
-- Loot of any other ilvl will have no special suffix in the loot log.

-- Lists
local memberMain = {}         -- {ID, name, points}
local memberOff = {}	
local trialMain = {}	
local trialOff = {}
local lootList = {}           -- {ID, item, count}
local assignList = {}         -- {ID, name}
local lootBtnList = {}        -- {ID, btn}
local assignBtnList = {}
local lastLootSeen = {}

-- Bid Specifics
local item = nil
local itemCount = 1
local curNum = 1
local currentBidder = nil
local bidderIndex = nil
local topBidder = nil
local channel = "RAID"
local minBid
local bidCategory

-- Booleans
local bTestMode = false     	  -- Is test mode enabled?
local bBidding = false      	  -- Is user currently bidding an item?
local bMult = false         	  -- More than one of the same item?
local bWarnedOOD = false		  -- Has the user been warned of an out of date version?
local bCorePanelExtended = false  -- Is the loot assignment portion shown?

-- Misc
local classColors = RAID_CLASS_COLORS
local selectedItem = nil
local selectedAssign = nil

--------------------)
---ACE3 Functions---)
--------------------)
function ARP:OnInitialize()
	-- Init Saved Variables (first time addon loads)
	if (ARP_Scale == nil) then ARP_Scale = 1 end
	
	-- Slash Commands
	local options = {
		name = "ARP",
		handler = ARP,
		type = "group",
		args = {
			toggle = {
				type = "execute",
				name = "Toggle Window",
				desc = "Shows/hides the main window (also available in game keybindings)",
				func = function() ARP_Toggle() end
			},
			reset = {
				type = "execute",
				name = "Value Reset",
				desc = "Resets, cancelling any bid you're currently handling",
				func = function() ARP_Reset() end
			},
			lastcall = {
				type = "execute",
				name = "Last Call",
				desc = "Provides a last call alert for your assigned item",
				func = function() 
					if (item or bTestMode) then ARP_LastCall() 
					else print("You have not been assigned an item!") end
				end
			},
			bid = {
				type = "execute",
				name = "Start Bidding",
				desc = "Starts bidding for your assigned item",
				func = function() 
					if (item or bTestMode) then ARP_StartBid()
					else print("You have not been assigned an item!") end 
				end
			},
			channel = {
				type = "select",
				name = "Channel Select",
				desc = function() return "Set channel in which to display bid information (current:|cFFFF0000 "..channel.."|r)" end,
				values = {raid="Raid", guild="Guild"},
				set = function(info, val) channel = strupper(val) print("Output channel changed to "..channel) end,
				get = function(info) return channel end,
				style = "dropdown"
			},	
			test = {
				type = "execute",
				name = "Test Mode",
				desc = "Enables/disables test mode",
				func = function() 
					if (bTestMode) then 
						bTestMode = false 
						print("Test mode disabled") 
					else 
						bTestMode = true 
						print("Test mode enabled") 
					end 
				end
			},		
			scale = {
				type = "range",
				name = "Scale",
				desc = function() 
					return "Sets the scale of the ARP windows (current:|cFFFF0000 "..ARP_Scale.."|r)" 
				end,
				min = 0.3,
				max = 3,
				get = function(info) return ARP_Scale end,
				set = function(info, val) ARP_Scale = val end
			},
			version = {
				type = "execute",
				name = "Get Versions",
				desc = "Displays a list of all users (in the guild) running the addon and the version they're running",
				func = function() 
					ARP.versionList = {} -- start with a fresh version list, when displaying it
					SendAddonMessage("ARP_VER_QUERY", nil, "GUILD") 
					self:ScheduleTimer("VersionCheckOutput", 2)
					print("|cFF33FF99Angry Raid Points version check running...|r ")
				end
			},
			reminder = {
				type = "execute",
				name = "Reminder",
				desc = "Displays current bid info and resends tell to the current bidder",
				func = function() 
					if (bBidding) then SendChatMessage("It is your bid! Min "..minBid..".", "WHISPER", nil, currentBidder)
					else print("You are not currently bidding an item!") end
				end
			},
		},
	}

	LibStub("AceConfig-3.0"):RegisterOptionsTable("Angry Raid Points", options, {"arp"})
end

function ARP:VersionCheckOutput()
	local versionliststr = ""
	for i,v in pairs(ARP.versionList) do
		versionliststr = versionliststr..v["name"].."-|cFFFF0000"..v["version"].."|r "
	end
	print(versionliststr) 
end

function ARP:OnEnable()
	local arpFrame = CreateFrame("Frame")
	arpFrame:RegisterEvent("CHAT_MSG_WHISPER")
	arpFrame:RegisterEvent("CHAT_MSG_BN_WHISPER")
	arpFrame:RegisterEvent("CHAT_MSG_ADDON")
	arpFrame:RegisterEvent("LOOT_OPENED")
	arpFrame:RegisterEvent("PARTY_LOOT_METHOD_CHANGED")
	arpFrame:RegisterEvent("GROUP_ROSTER_UPDATE")
	RegisterAddonMessagePrefix("ARP_LOG")
	RegisterAddonMessagePrefix("ARP_ASSIGN")
	RegisterAddonMessagePrefix("ARP_ASSIGN_MULT")	
	RegisterAddonMessagePrefix("ARP_QUERY")
	RegisterAddonMessagePrefix("ARP_READY")
	RegisterAddonMessagePrefix("ARP_VER_QUERY")
	RegisterAddonMessagePrefix("ARP_VERSION")
	
	arpFrame:SetScript("OnEvent", ARP_OnEvent)
	ARP_CreateCoreFrame()
	
	self:ScheduleTimer("AfterEnable", 5)
end

function ARP:AfterEnable()
	SendAddonMessage("ARP_VER_QUERY", nil, "GUILD")
	if (IsInRaid()) then SendAddonMessage("ARP_READY", nil, "RAID") end
end

--------------------------)
---Core Panel Functions---)
--------------------------) 
function ARP_CopyLootLog()
	local lootLog = ARP.corePanel.lootLog.text
	local copyFrame = ARP.corePanel.lootLog.copyFrame.text
	local firstline = true
	
	copyFrame:SetText("")
	for i=lootLog:GetNumMessages(), 1, -1 do
		local msg = lootLog:GetMessageInfo(i)
		if (strmatch(msg, "= DE$") == nil) then
			if firstline then 
				firstline = false
			else
				copyFrame:Insert("\n")
			end
			copyFrame:Insert( msg )
		end
	end
	copyFrame:GetParent():Show()
	copyFrame:Show()
	copyFrame:SetFocus()
	copyFrame:HighlightText(0)
	
	copyFrame:SetScript("OnEscapePressed", function(copyFrame)
		copyFrame:HighlightText(copyFrame:GetCursorPosition(), copyFrame:GetCursorPosition())
		copyFrame:ClearFocus()
	end)
end

function ARP_CreateCoreFrame()
	local frame
	local button
	local title
	local text
	local dropdown
	local label	-- Predeclare variables to make future editing easier

	-- Core Panel
	frame = CreateFrame("Frame", "ARPCorePanel", UIParent)
	frame:SetSize(475, 350)
	frame:SetScale(ARP_Scale)
	frame:SetPoint("CENTER")
	frame:SetFrameStrata("HIGH")
	frame:SetFrameLevel(1)
	frame:SetBackdrop( {	bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",
							edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
							tile = true,
							tileSize = 32,
							edgeSize = 24,
							insets = { left = 5, right = 6, top = 6, bottom = 5 } } )
	frame:EnableMouse(true)
	title = frame:CreateTitleRegion()
	title:SetAllPoints()
	frame:SetScript("OnShow", ARP_CorePanel_OnShow)
	frame:SetScript("OnUpdate", ARP_CorePanel_OnUpdate)
	tinsert(UISpecialFrames, "ARPCorePanel")
	frame:Hide()
	ARP.corePanel = frame
	
	-- Core Panel Title Texture
	local texture = ARP.corePanel:CreateTexture()
	texture:SetTexture("Interface\\Addons\\AngryRaidPoints\\Angry_Logo.tga")
	-- texture:SetPoint("TOP", ARP.corePanel, "TOP", -15, 0)
	texture:SetPoint("TOP", ARP.corePanel, "TOPLEFT", 220, 0)
	
	label = ARP.corePanel:CreateFontString(nil, "OVERLAY")
	label:SetPoint("LEFT", texture, "RIGHT", 0, 5)
	label:SetJustifyH("LEFT")
	label:SetFont("Interface\\Addons\\AngryRaidPoints\\BlackChancery.ttf", 18, "OUTLINE")
	label:SetTextColor(1, 0.82, 0, 1)
	label:SetText("RP")
	
	label = ARP.corePanel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
	label:SetPoint("TOPLEFT", texture, "BOTTOMRIGHT", -15, 20)
	label:SetJustifyH("LEFT")
	local versiontext = ARP.VERSION
	if (versiontext:sub(1,1) == "@") then versiontext = "dev" end
	label:SetText(versiontext)
	
	-- Assigned Item Label
	label = ARP.corePanel:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	label:SetPoint("LEFT", ARP.corePanel, "LEFT", 10, 100)
	label:SetJustifyH("LEFT")
	label:SetText("ITEM")
	ARP.corePanel.itemLabel = label
	
	-- Assigned Item Text
	text = ARP.corePanel:CreateFontString(nil, "OVERLAY")
	text:SetPoint("LEFT", ARP.corePanel.itemLabel, "LEFT", 5, -20)
	text:SetJustifyH("LEFT")
	text:SetFont("Fonts\\FRIZQT__.TTF", 12, "OUTLINE")
	ARP.corePanel.itemText = text
	
	-- Channel Output Label
	label = ARP.corePanel:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	label:SetPoint("LEFT", ARP.corePanel, "LEFT", 10, 50)
	label:SetJustifyH("LEFT")
	label:SetText("CHANNEL OUTPUT")
	ARP.corePanel.channelLabel = label
	
	-- Channel Output Dropdown
	dropdown = CreateFrame("Frame", "ChannelOutputDropdown", ARP.corePanel, "UIDropDownMenuTemplate")
	dropdown:SetPoint("LEFT", ARP.corePanel.channelLabel, "LEFT", -10, -25)
	ARP.corePanel.channelDropdown = dropdown
	
	-- Current Bidder Label
	label = ARP.corePanel:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	label:SetPoint("LEFT", ARP.corePanel, "LEFT", 10, 0)
	label:SetJustifyH("LEFT")
	label:SetText("CURRENT BIDDER")
	ARP.corePanel.curBidderLabel = label
	
	-- Current Bidder Text
	text = ARP.corePanel:CreateFontString(nil, "OVERLAY")
	text:SetPoint("LEFT", ARP.corePanel.curBidderLabel, "LEFT", 5, -20)
	text:SetJustifyH("LEFT")
	text:SetFont("Fonts\\FRIZQT__.TTF", 12, "OUTLINE")
	ARP.corePanel.curBidderText = text
	
	-- Top Bidder Label
	label = ARP.corePanel:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	label:SetPoint("LEFT", ARP.corePanel, "LEFT", 10, -50)
	label:SetJustifyH("LEFT")
	label:SetText("TOP BIDDER")
	ARP.corePanel.topBidderLabel = label
	
	-- Top Bidder Text
	text = ARP.corePanel:CreateFontString(nil, "OVERLAY")
	text:SetPoint("LEFT", ARP.corePanel.topBidderLabel, "LEFT", 5, -20)
	text:SetJustifyH("LEFT")
	text:SetFont("Fonts\\FRIZQT__.TTF", 12, "OUTLINE")
	ARP.corePanel.topBidderText = text
	
	-- Core Frame Close Button
	button = CreateFrame("Button", "CoreCloseBtn", ARP.corePanel, "UIPanelButtonTemplate")
	button:SetSize(25, 25)
	button:SetPoint("TOPRIGHT", ARP.corePanel, "TOPRIGHT", -5, -6)
	button:SetText("X")
	button:SetScript("OnClick", function()
		ARP.corePanel:Hide()
	end)
	ARP.corePanel.closeBtn = button
	
	-- Core Frame Start Bid Button
	button = CreateFrame("Button", "CoreBidBtn", ARP.corePanel, "UIPanelButtonTemplate")
	button:SetSize(125, 30)
	button:SetPoint("BOTTOM", ARP.corePanel, "BOTTOMLEFT", 235, 10)
	button:SetText("Start Bid")
	button:Disable()
	button:SetScript("OnClick", function()
		ARP_StartBid()
	end)
	ARP.corePanel.startBidBtn = button
	
	-- Core Frame Loot Assignment Button
	button = CreateFrame("Button", "CoreMLPanelShowBtn", ARP.corePanel, "UIPanelButtonTemplate")
	button:SetSize(125, 30)
	button:SetPoint("BOTTOMRIGHT", ARP.corePanel, "BOTTOMLEFT", 460, 10)
	button:SetText("Loot Assignment")
	button:Disable()
	button:SetScript("OnClick", function()
		if (bCorePanelExtended) then 
			bCorePanelExtended = false
		else 
			bCorePanelExtended = true 
			ARP_MLPanel_Update()
		end
	end)
	ARP.corePanel.lootAssignBtn = button
	
	-- Core Frame Last Call Button
	button = CreateFrame("Button", "CoreBidListBtn", ARP.corePanel, "UIPanelButtonTemplate")
	button:SetSize(125, 30)
	button:SetPoint("BOTTOMLEFT", ARP.corePanel, "BOTTOMLEFT", 10, 10)
	button:SetText("Last Call")
	button:Disable()
	button:SetScript("OnClick", function()
		ARP_LastCall()
	end)
	ARP.corePanel.lastCallBtn = button
	
	-- Core Frame Reminder Button
	button = CreateFrame("Button", "CoreReminderBtn", ARP.corePanel, "UIPanelButtonTemplate")
	button:SetSize(125, 30)
	button:SetPoint("BOTTOM", ARP.corePanel.lastCallBtn, "TOP", 0, 0)
	button:SetText("Reminder")
	button:Disable()
	button:SetScript("OnClick", function()
		SendChatMessage("It is your bid! Min "..minBid..".", "WHISPER", nil, currentBidder)			
	end)
	ARP.corePanel.reminderBtn = button
	
	-- Loot Log Container
	frame = CreateFrame("Frame", "CoreLootLogContainer", ARP.corePanel)
	frame:SetSize(300, 150)
	frame:SetPoint("RIGHT", ARP.corePanel, "LEFT", 460, -20)
	frame:SetBackdrop( {	bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background-Dark",
							edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
							tile = true,
							tileSize = 32,
							edgeSize = 24,
							insets = { left = 5, right = 6, top = 6, bottom = 5 } } )
	ARP.corePanel.lootLog = frame
	
	-- Loot Log Label Text
	label = ARP.corePanel.lootLog:CreateFontString("LootLogLabel", "OVERLAY", "GameFontNormal")
	label:SetPoint("BOTTOM", ARP.corePanel.lootLog, "TOP")
	label:SetText("LOOT LOG")
	ARP.corePanel.lootLog.label = label
	
	-- Loot Log Message Box
	frame = CreateFrame("ScrollingMessageFrame", "LootLogText", ARP.corePanel.lootLog)
	frame:SetSize(284, 134)
	frame:SetPoint("TOPLEFT", ARP.corePanel.lootLog, "TOPLEFT", 8, -8)
	frame:SetJustifyH("LEFT")
	frame:SetFadeDuration(0)
	frame:SetFading(false)
	frame:SetInsertMode("TOP")
	frame:SetFontObject("GameFontNormalSmall")
	ARP.corePanel.lootLog.text = frame
	
	-- Loot Log Copy Container
	frame = CreateFrame("Frame", "LootLogCopyFrame", ARP.corePanel.lootLog)
	frame:SetSize(300, 150)
	frame:SetPoint("CENTER", ARP.corePanel, "TOP", 0, 0)
	frame:SetFrameLevel(ARP.corePanel.lootLog:GetFrameLevel()+2)
	frame:SetBackdrop( {	bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background-Dark",
							edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
							tile = true,
							tileSize = 32,
							edgeSize = 24,
							insets = { left = 5, right = 6, top = 6, bottom = 5 } } )
	frame:EnableMouse(true)
	title = frame:CreateTitleRegion()
	title:SetAllPoints()
	frame:Hide()
	ARP.corePanel.lootLog.copyFrame = frame
	
	-- Loot Log Copy Container Text
	frame = CreateFrame("EditBox", "LootLogCopyFrameText", ARP.corePanel.lootLog.copyFrame)
	frame:SetSize(284, 134)
	frame:SetPoint("TOPLEFT", ARP.corePanel.lootLog.copyFrame, "TOPLEFT", 8, -8)
	frame:SetJustifyH("LEFT")
	frame:SetAutoFocus(false)
	frame:SetMultiLine(true)
	frame:SetMaxLetters(480)
	frame:SetFontObject("GameFontNormalSmall")	
	frame:Hide()
	ARP.corePanel.lootLog.copyFrame.text = frame
	
	-- Loot Log Copy Container Close Button
	button = CreateFrame("Button", "CopyContainerCloseBtn", ARP.corePanel.lootLog.copyFrame, "UIPanelButtonTemplate")
	button:SetSize(150, 30)
	button:SetPoint("CENTER", ARP.corePanel.lootLog.copyFrame, "BOTTOM", 0, -15)
	button:SetText("Close")
	button:SetScript("OnClick", function()
		ARP.corePanel.lootLog.copyFrame:Hide()
	end)
	
	-- Copy Loot Log Button
	button = CreateFrame("Button", "CopyLootLogBtn", ARP.corePanel.lootLog, "UIPanelButtonTemplate")
	button:SetSize(75, 30)
	button:SetPoint("CENTER", ARP.corePanel.lootLog, "BOTTOM", -75, -15)
	button:SetText("Copy")
	button:SetScript("OnClick", function()
		if (not ARP.corePanel.lootLog.copyFrame:IsShown()) then
			ARP_CopyLootLog()
		end
	end)
	ARP.corePanel.lootLog.copyBtn = button
	
	-- Clear Loot Log Button
	button = CreateFrame("Button", "ClearLootLogBtn", ARP.corePanel.lootLog, "UIPanelButtonTemplate")
	button:SetSize(75, 30)
	button:SetPoint("CENTER", ARP.corePanel.lootLog, "BOTTOM", 75, -15)
	button:SetText("Clear")
	button:SetScript("OnClick", function()
		ARP.corePanel.lootLog.text:Clear()
	end)
	ARP.corePanel.lootLog.clearBtn = button
	
	-----------------------------)
	---Loot Assignment Portion---)
	-----------------------------)
	
	-- Loot Assignment Item List Container
	frame = CreateFrame("Frame", "MLItemList", ARP.corePanel)
	frame:SetSize(300, 120)
	frame:SetPoint("TOP", ARP.corePanel, "TOPRIGHT", -187.5, -20)
	frame:SetBackdrop( {	bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background-Dark",
							edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
							tile = true,
							tileSize = 32,
							edgeSize = 24,
							insets = { left = 5, right = 6, top = 6, bottom = 5 } } )
	frame:Hide()
	ARP.corePanel.itemList = frame
	
	-- Loot Assignment Item List Label
	label = ARP.corePanel.itemList:CreateFontString("ItemListLabel", "OVERLAY", "GameFontNormal")
	label:SetPoint("BOTTOM", ARP.corePanel.itemList, "TOP")
	label:SetJustifyH("CENTER")
	label:SetText("ITEM LIST")
	ARP.corePanel.itemList.label = label
	
	-- Loot Assignment Item List Reload Button
	button = CreateFrame("Button", "MLLootClearBtn", ARP.corePanel.itemList, "UIPanelButtonTemplate")
	button:SetSize(75, 30)
	button:SetPoint("TOP", ARP.corePanel.itemList, "BOTTOM", 0, 0)
	button:SetText("Reload")
	button:SetScript("OnClick", function()
		ARP_MLPanel_ClearLoot()
	end)
	ARP.corePanel.itemList.clearBtn = button
	
	-- Loot Assignment Assignment List
	frame = CreateFrame("Frame", "MLAssignList", ARP.corePanel)
	frame:SetSize(150, 120)
	frame:SetPoint("BOTTOM", ARP.corePanel, "BOTTOMRIGHT", -187.5, 40)
	frame:SetBackdrop( {	bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background-Dark",
							edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
							tile = true,
							tileSize = 32,
							edgeSize = 24,
							insets = { left = 5, right = 6, top = 6, bottom = 5 } } )
	frame:Hide()
	ARP.corePanel.assignList = frame
	
	-- Loot Assignment Assignment List Label
	label = ARP.corePanel.assignList:CreateFontString("AssignListLabel", "OVERLAY", "GameFontNormal")
	label:SetPoint("BOTTOM", ARP.corePanel.assignList, "TOP")
	label:SetJustifyH("CENTER")
	label:SetText("ASSIGN LIST")
	ARP.corePanel.assignList.label = label	
	
	-- Loot Assignment Assignment List Loot Assignment Button (Hahahaha)
	button = CreateFrame("Button", "MLAssignBtn", ARP.corePanel.assignList, "UIPanelButtonTemplate")
	button:SetSize(75, 30)
	button:SetPoint("TOP", ARP.corePanel.assignList, "BOTTOM", 0, 0)
	button:SetText("Assign")
	button:SetScript("OnClick", function()
		if (selectedItem and selectedAssign) then
			ARP_MLPanel_AssignLoot()
			ARP_MLPanel_Update()
		end
	end)
	ARP.corePanel.assignBtn = button
end

function ARP_CorePanel_OnShow(self)
	self:SetScale(ARP_Scale)
	SendAddonMessage("ARP_QUERY", nil, "RAID")
	if (itemCount > 1) then	ARP.corePanel.itemText:SetText(item.."x"..itemCount)
	else ARP.corePanel.itemText:SetText(item) end
	ARP.corePanel.curBidderText:SetText(currentBidder)
	if (minBid ~= nil and topBidder ~= nil) then
		ARP.corePanel.topBidderText:SetText(topBidder) 
	end
	
	local function itemOnClick(self)
		local selected = self.value
		channel = selected
		UIDropDownMenu_SetSelectedValue(ARP.corePanel.channelDropdown, selected)
	end
	UIDropDownMenu_Initialize(ARP.corePanel.channelDropdown, function()
		local info = UIDropDownMenu_CreateInfo()
		local channels = {"RAID", "GUILD"}
		for i, chnl in next, channels do
			info.text = channels[i]
			info.value = chnl
			info.func = itemOnClick
			info.checked = chnl == channel
			UIDropDownMenu_AddButton(info)
		end
	end)
	UIDropDownMenu_SetSelectedValue(ARP.corePanel.channelDropdown, channel)
	UIDropDownMenu_SetWidth(ARP.corePanel.channelDropdown, 80)
	UIDropDownMenu_JustifyText(ARP.corePanel.channelDropdown, "LEFT")
	
	if (item or bTestMode) then
		ARP.corePanel.startBidBtn:Enable()
		ARP.corePanel.lastCallBtn:Enable()
	else 
		ARP.corePanel.startBidBtn:Disable()
		ARP.corePanel.lastCallBtn:Disable()
	end
	
	ARP.corePanel:SetScript("OnShow", nil)	
end

function ARP_CorePanel_OnUpdate(self, elapsed)
	if (not bCorePanelExtended) then	
		ARP.corePanel:SetSize(470, 350)
		ARP.corePanel.itemList:Hide()
		ARP.corePanel.assignList:Hide()
	else
		ARP.corePanel:SetSize(810, 350)
		ARP.corePanel.itemList:Show()
		ARP.corePanel.assignList:Show()
	end
	
	self:SetScale(ARP_Scale)
	if (itemCount > 1) then ARP.corePanel.itemText:SetText(item.."x"..itemCount)
	else ARP.corePanel.itemText:SetText(item) end
	
	UIDropDownMenu_SetSelectedValue(ARP.corePanel.channelDropdown, channel)
	
	if (currentBidder ~= nil) then
		if (UnitName("player") == currentBidder or UnitInRaid(currentBidder)) then
			local color = classColors[strupper(UnitClass(currentBidder))]
			if color then
				ARP.corePanel.curBidderText:SetText("|c"..color.colorStr..currentBidder.."|r")
			else
				ARP.corePanel.curBidderText:SetText(currentBidder)
			end
		else 
			ARP.corePanel.curBidderText:SetText(currentBidder) 
		end
	else 
		ARP.corePanel.curBidderText:SetText(nil)
	end
	
	if (minBid ~= nil and topBidder ~= nil) then
		if (UnitName("player") == topBidder or UnitInRaid(topBidder)) then
			local color = classColors[strupper(UnitClass(topBidder))]
			if color then
				ARP.corePanel.topBidderText:SetText("|c"..color.colorStr..topBidder.."|r at "..(minBid-1).." points")
			else
				ARP.corePanel.topBidderText:SetText(topBidder.." at "..(minBid-1).." points")
			end
		else
			ARP.corePanel.topBidderText:SetText(topBidder.." at "..(minBid-1).." points") 
		end
	else 
		ARP.corePanel.topBidderText:SetText(nil)
	end
	
	-- Button enable/disable checks
	if (ARP.corePanel.lootLog.text:GetNumMessages() > 0) then
		ARP.corePanel.lootLog.copyBtn:Enable()
		ARP.corePanel.lootLog.clearBtn:Enable()
	else
		ARP.corePanel.lootLog.copyBtn:Disable()
		ARP.corePanel.lootLog.clearBtn:Disable()
	end
	
	if (item or bTestMode) then
		ARP.corePanel.startBidBtn:Enable()
		ARP.corePanel.lastCallBtn:Enable()
	else
		ARP.corePanel.startBidBtn:Disable()
		ARP.corePanel.lastCallBtn:Disable()
	end
	
	if (bBidding) then ARP.corePanel.reminderBtn:Enable()
	else ARP.corePanel.reminderBtn:Disable() end

	if (IsInRaid()) then
		local mlName
		local method, _, mlID = GetLootMethod()
		if (method == "master") then mlName = GetRaidRosterInfo(mlID) end
		if ((method ~= "master") or (mlName == UnitName("player")) or bTestMode) then ARP.corePanel.lootAssignBtn:Enable()
		else ARP.corePanel.lootAssignBtn:Disable() end
	else ARP.corePanel.lootAssignBtn:Disable()
	end
end

function ARP_Toggle()
	if ARP.corePanel:IsShown() then 
		ARP.corePanel:Hide() 
	else 
		ARP.corePanel:Show() 
	end
end

---------------------------------------)
---Loot Assignment Section Functions---)
---------------------------------------)

function ARP_MLPanel_Update()
	if (not bCorePanelExtended) then return end
	for i,v in pairs(lootBtnList) do v["btn"]:Hide() end
	lootBtnList = {}
	for i=1, #lootList do
		local button = CreateFrame("CheckButton", "LootButton"..i, ARP.corePanel.itemList, "UIRadioButtonTemplate")
		button.text = select(2, GetItemInfo(lootList[i]["item"]))
		button.count = lootList[i]["count"]
		local font = button:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
		font:SetPoint("LEFT", button, "RIGHT", 5, 1)
		button:SetFontString(font)
		button:SetHitRectInsets(0, -225, 0, 0)
		if (lootList[i]["count"] > 1) then button:SetText(button.text.."x"..lootList[i]["count"])
		else button:SetText(button.text) end
		if (i > 1) then 
			button:SetPoint("TOPLEFT", ARP.corePanel.itemList.prevBtn, "BOTTOMLEFT", 0, 0)
		else button:SetPoint("TOPLEFT", ARP.corePanel.itemList, "TOPLEFT", 5, -5) end
		button:SetScript("OnEnter", function(...) 
				GameTooltip:SetOwner(ARP.corePanel.lastCallBtn, "ANCHOR_TOPRIGHT")
				GameTooltip:SetHyperlink(button.text)
				GameTooltip:Show()
				end)
		button:SetScript("OnLeave", function(...) GameTooltip:Hide() end)
		button:SetScript("OnClick", function(self, button, up)
			if IsModifiedClick() then
				-- Re-toggle the actual button then do the proper action
				local checked = self:GetChecked()
				self:SetChecked(not checked)
				HandleModifiedItemClick(self:GetText())
			end
			if (self:GetChecked()) then
				selectedItem = i
				ARP_MLPanel_SelectItem(i)
			else 
				selectedItem = nil
			end
		end)
		button:Show()
		ARP.corePanel.itemList.prevBtn = button
		table.insert(lootBtnList, {btn = button})
	end
	
	for i,v in pairs(assignBtnList) do v["btn"]:Hide() end
	assignBtnList = {}
	for i=1, #assignList do
		local button = CreateFrame("CheckButton", "AssignButton"..i, ARP.corePanel.assignList, "UIRadioButtonTemplate")
		button.text = assignList[i]["name"]
		local font = button:CreateFontString(nil, "OVERLAY", "GameFontNormal")
		font:SetPoint("LEFT", button, "RIGHT", 5, 1)
		button:SetFontString(font)
		button:SetHitRectInsets(0, -225, 0, 0)
		button:SetText(button.text)
		if (i > 1) then	button:SetPoint("TOPLEFT", ARP.corePanel.assignList.prevBtn, "BOTTOMLEFT", 0, 0)
		else button:SetPoint("TOPLEFT", ARP.corePanel.assignList, "TOPLEFT", 5, -5) end
		button:SetScript("OnClick", function(self, button, up)
			if (self:GetChecked()) then
				selectedAssign = i
				ARP_MLPanel_SelectAssign(i)
			else selectedAssign = nil end
		end)
		button:Show()
		ARP.corePanel.assignList.prevBtn = button
		table.insert(assignBtnList, {btn = button})
	end

end

function ARP_MLPanel_AssignLoot()
	SendAddonMessage("ARP_ASSIGN", lootBtnList[selectedItem]["btn"].text, "WHISPER", assignList[selectedAssign]["name"])
	if (lootBtnList[selectedItem]["btn"].count > 1) then 
		SendChatMessage(lootBtnList[selectedItem]["btn"].text.."x"..lootBtnList[selectedItem]["btn"].count..": tells to "..assignList[selectedAssign]["name"], channel)
		SendAddonMessage("ARP_ASSIGN_MULT", lootBtnList[selectedItem]["btn"].count, "WHISPER", assignList[selectedAssign]["name"])
	else SendChatMessage(lootBtnList[selectedItem]["btn"].text..": tells to "..assignList[selectedAssign]["name"], channel) end
	table.remove(lootList, selectedItem)
	table.remove(assignList, selectedAssign)
	selectedItem = nil
	selectedAssign = nil
end

function ARP_MLPanel_ClearLoot()
	lootList = {}
	lastLootSeen = {}
	selectedItem = nil
	ARP_AttemptLootScan()
	ARP_MLPanel_Update()
end

function ARP_AttemptLootScan()
	local thisLootSeen = {}

	for i=1, GetNumLootItems() do
		if (GetLootSlotType(i) == 1) then
			local _, itemID = strsplit(":", GetLootSlotLink(i))
			local _, _, rarity = GetItemInfo(itemID)
			if (rarity == 4) then table.insert(thisLootSeen, itemID) end
		end
	end
	
	if (#thisLootSeen > 0) then
		local newloot = false
		local found = false
		
		for i=1, #thisLootSeen do
			found = false
			
			for _,v in pairs(lastLootSeen) do
				if v == thisLootSeen[i] then
					found = true
					break
				end
			end
		
			if found == false then
				newloot = true
				break
			end
		end
		
		if (newloot) then 
			ARP_Cleanup_AssignList()
			lastLootSeen = {}
			for i,v in pairs(thisLootSeen) do
				table.insert(lastLootSeen, v)
			end
			ARP_BuildItemList(thisLootSeen) 
		end
	end
end

function ARP_MLPanel_SelectItem(selected)
	for i,v in pairs(lootBtnList) do
		if (i ~= selected) then v["btn"]:SetChecked(false) end
	end
end

function ARP_MLPanel_SelectAssign(selected)
	for i,v in pairs(assignBtnList) do
		if (i ~= selected) then v["btn"]:SetChecked(false) end
	end
end

--------------------)
---Event Handling---)
--------------------)
function ARP_OnEvent(self, event, ...)
	-- Whisper
	if (event == "CHAT_MSG_WHISPER") then
		local msg, sender = ...
		if (UnitInRaid(sender)) then ARP_HandleWhisper(msg, sender)
		else return end
		
	-- Battle.net Whisper
	elseif (event == "CHAT_MSG_BN_WHISPER") then
		local msg, sender, _, _, _, _, _, _, _, _, _, _, presenceID = ...
		local _, onlineBnetFriends = BNGetNumFriends()
		for i=1, onlineBnetFriends do
			local friendID, _, _, _, toonName, _, client = BNGetFriendInfo(i)
			if (client == BNET_CLIENT_WOW and presenceID == friendID) then
				if (UnitInRaid(toonName)) then ARP_HandleWhisper(msg, toonName) break
				else return end
			end
		end
		
	-- Addon communication
	elseif (event == "CHAT_MSG_ADDON") then
		local prefix, message, _, sender = ...
		
		-- Add a result to the loot log
		if (prefix == "ARP_LOG") then
			ARP.corePanel.lootLog.text:AddMessage(message)	
			
		-- User has been assigned an item
		elseif (prefix == "ARP_ASSIGN") then
			item = message			
			
		-- User has been assigned an item with duplicates ([Item]x3, etc.)
		elseif (prefix == "ARP_ASSIGN_MULT") then
			itemCount = tonumber(message)
			bMult = true
			
		-- Is user ready to be assigned an item?
		elseif (prefix == "ARP_QUERY") then
			if (not item) then SendAddonMessage("ARP_READY", nil, "RAID") end
			
		-- Broadcast availability for being assigned an item
		elseif (prefix == "ARP_READY") then
			local found = false
			local i = 1
			while i <= #assignList do
				if sender == assignList[i]["name"] then
					found = true
					if not UnitInRaid(sender) then
						table.remove(assignList, i)
					else
						i = i + 1
					end
				else
					i = i + 1
				end
			end
			if (not found) and UnitInRaid(sender) then 
				table.insert(assignList, {ID = #assignList+1, name = sender}) 
			end
			ARP_MLPanel_Update()
			
		-- Another user has requested this user's current version
		elseif (prefix == "ARP_VER_QUERY") then
			local revToSend
			local verToSend
			if ARP.REVISION:sub(1,1) == "@" then revToSend = "dev" else revToSend = "r"..ARP.REVISION end
			if ARP.VERSION:sub(1,1) == "@" then verToSend = "dev" else verToSend = ARP.VERSION end
			SendAddonMessage("ARP_VERSION", revToSend.." "..verToSend, "GUILD")
			
		-- Received version broadcast from other users
		elseif (prefix == "ARP_VERSION") then
			local localRev, rev, ver
			
			if ARP.REVISION:sub(1,1) == "@" then localRev = nil else localRev = tonumber(ARP.REVISION) end

			if (message:sub(1,1) == "@") then -- Handle old format
				rev = nil
				ver = "dev"
			else
				local m1, m2 = strmatch(message, "^([^ ]*) (.*)$")
				if (m1 ~= nil and m2 ~= nil) then
					if (m1 == "dev") then rev = nil else rev = tonumber(m1:sub(2)) end
					ver = m2
				else -- Handle old format
					rev = tonumber(message:sub(2))
					ver = message
				end
			end
			
			if ((localRev ~= nil) and (rev ~= nil) and (rev > localRev) and (not bWarnedOOD)) then 
				print("|cFF33FF99Angry Raid Points|r: Your version of AngryRaidPoints is out of date! Download the latest version from www.wowace.com.")
				bWarnedOOD = true
			end

			local found = false
			for i,v in pairs(ARP.versionList) do
				if (v["name"] == sender) then
					v["version"] = ver
					found = true
				end
			end
			if (not found) then table.insert(ARP.versionList, {name = sender, version = ver}) end
		end
		
	-- If player is the master looter (or if master loot isn't set), generate the loot into an assignable list
	elseif ( IsInRaid() and (event == "LOOT_OPENED") ) then
		local mlName
		local method, _, mlID = GetLootMethod()
		if (method == "master") then mlName = GetRaidRosterInfo(mlID) end
		if (method ~= "master" or (mlName == UnitName("player")) or bTestMode) then ARP_AttemptLootScan() end
	elseif ( IsInRaid() and (event == "PARTY_LOOT_METHOD_CHANGED") ) then
		bCorePanelExtended = false
	elseif ( IsInRaid() and (event == "GROUP_ROSTER_UPDATE") ) then
		ARP_Cleanup_AssignList()
	end
end

function ARP_Cleanup_AssignList() 
	local changes = false
	-- Clean up any outdated entries from assignList
	local al = 1
	while al <= #assignList do
		if (not UnitInRaid(assignList[al]["name"])) or (not UnitIsConnected(assignList[al]["name"])) then
			table.remove(assignList, al)
			changes = true
		else
			al = al + 1
		end
	end
	if changes then ARP_MLPanel_Update() end
end

function ARP_GetGuildRank(player)
	for i = 1, GetNumGuildMembers() do
		local name, _, rankIndex = GetGuildRosterInfo(i)
		if name and (name == player) then
			return rankIndex
		end
	end
	return -1
end

function ARP_HandleWhisper(msg, sender)
	msg = { strsplit(" ", strlower(msg)) }
	local points, spec = "", ""
	if (not bBidding) then 
		if ( (strfind(msg[1], "%a+") and msg[2] == nil) or (strfind(msg[1], "%a+") and strfind(msg[2], "%a+")) ) then return end
		if ( strfind(msg[1], "^-*%d+$") and (msg[2] == "" or msg[2] == nil or strfind(msg[2], "^%a+$")) ) then
			points = msg[1]
			spec = msg[2]
		elseif ( strfind(msg[1], "^%a+$") and strfind(msg[2], "^-*%d+$") ) then
			points = msg[2]
			spec = msg[1]
		end
	else points = msg[1]
	end
	if (points == "pass" or points == "yup" or strfind(points, "^-*%d+$") or ((points:sub(1,1) == "+") and (tonumber(points:sub(2)) > 0)))
		and (item or bTestMode) then
		if (bBidding) then
			if (sender == currentBidder) then
				local list = ARP_GetList(bidCategory)
				if (points ~= "pass" and list[bidderIndex]["points"] < minBid) then
					ARP_UpdateBid(-1)
				else
					ARP_UpdateBid(points)
				end
			end
		else
			points = tonumber(points)
			local response
			
			-- Generalize spec
			if (spec == "ms" or spec == "" or spec == nil) then spec = "main"
			elseif (spec == "os") then spec = "off"
			end
			
			if (spec ~= "main" and spec ~= "off") then
				response = "Invalid spec! Please resend tell. Valid options are main/ms or off/os. You can also leave spec blank to default to main spec."
			else
				local guildRankIndex = ARP_GetGuildRank(sender)
				if (guildRankIndex >= 0 and guildRankIndex <= rankMember and guildRankIndex ~= rankMemberException) then
					if (spec == "main") then ARP_UpdateList(memberMain, sender, points, memberOff)
					else ARP_UpdateList(memberOff, sender, points, memberMain) end
					response = "Tell received! You have ["..points.."] points and are bidding for your ["..spec.."] spec."				
				elseif (guildRankIndex == rankTrial) then
					if (spec == "main") then ARP_UpdateList(trialMain, sender, points, trialOff)
					else ARP_UpdateList(trialOff, sender, points, trialMain) end
					response = "Tell received! You have ["..points.."] points and are bidding for your ["..spec.."] spec."				
				elseif (guildRankIndex == -1) then
					response = "Unable to obtain your guild rank currently. Try again by re-sending your tell."
				else
					response = "Your guild rank is invalid. Tell ignored."
				end
			end
			if (sender ~= UnitName("player")) then SendChatMessage(response, "WHISPER", nil, sender)
			else print(response) end
		end
	end	
end

-----------------------)
---Bidding Functions---)
-----------------------)
function ARP_BidItem(channel)
	local msg
	if (bMult) then msg = (item.." #"..curNum..": Starting bid! ")
	else msg = (item..": Starting bid! ") end
	local name, response
	
	memberMain = ARP_Sort(memberMain)
	memberOff = ARP_Sort(memberOff)
	
	if (memberMain[1]) then
		name = memberMain[1]["name"]
		minBid = minBidMMS
		bidCategory = "mm"
	elseif (memberOff[1]) then
		name = memberOff[1]["name"]
		minBid = minBidMOS
		bidCategory = "mo"
	elseif (trialMain[1]) then
		name = trialMain[1]["name"]
		minBid = minBidTMS
		bidCategory = "tm"
	elseif (trialOff[1]) then
		name = trialOff[1]["name"]
		minBid = minBidTOS
		bidCategory = "to"
	end
	
	SendChatMessage(msg..name.." min "..minBid, channel)
	SendChatMessage("It is your bid! Min "..minBid..".", "WHISPER", nil, name)
	currentBidder = name
	bidderIndex = 1
	bBidding = true
end

function ARP_BuildBidString(itemLink)
	local response
	if (itemCount > 1) then response = (itemLink.."x"..itemCount..": ")
	else response = (itemLink..": ") end
		
	memberMain = ARP_Sort(memberMain)
	for i,v in pairs(memberMain) do
		if (i == #memberMain) then response = (response..v["name"].." "..v["points"].." ")
		else response = (response..v["name"].." "..v["points"]..", ") end
	end
	memberOff = ARP_Sort(memberOff)
	for i,v in pairs(memberOff) do
		if (i == 1) then response = (response.."(") end
		if (i == #memberOff) then response = (response..v["name"].." "..v["points"].." OS) ")
		else response = (response..v["name"].." "..v["points"]..", ") end
	end
	trialMain = ARP_Sort(trialMain)
	for i,v in pairs(trialMain) do
		if (i == 1) then response = (response.."(") end
		if (i == #trialMain) then response = (response..v["name"].." "..v["points"].." Trial) ")
		else response = (response..v["name"].." "..v["points"]..", ") end
	end
	trialOff = ARP_Sort(trialOff)
	for i,v in pairs(trialOff) do
		if (i == 1) then response = (response.."(") end
		if (i == #trialOff) then response = (response..v["name"].." "..v["points"].." Trial OS) ")
		else response = (response..v["name"].." "..v["points"]..", ") end
	end
	response = response.."LAST CALL"
	
	return response	
end

function ARP_BuildItemList(itemIDList)
	lootList = {}
	
	for i,v in pairs(itemIDList) do
		local found = false
		for x,y in pairs(lootList) do
			if (y["item"] == v) then
				y["count"] = y["count"]+1
				found = true
			end
		end
		if (not found) then table.insert(lootList, {ID = #lootList+1, item = v, count = 1}) end
	end
	
	if (bCorePanelExtended) then ARP_MLPanel_Update() end
end

function ARP_LastCall()
	if (bBidding) then
		print("Your bid is already in progress!")
		return
	end
	if (not item and bTestMode) then
		table.insert(lootList, {ID = #lootList+1, item = 19019, count = 2})
		table.insert(lootList, {ID = #lootList+1, item = 87166, count = 1})
		ARP_MLPanel_Update()
	end
	if (item) then
		local response = ARP_BuildBidString(item)
		SendChatMessage(response, channel)
	end
end

function ARP_EndBid(winner, winAmt)
	if (winner == nil or winner == "None") then		-- Everyone passed
		if (bidCategory == "mm") then
			if (memberOff[1]) then
				bidCategory = "mo"
				bidderIndex = 1
				currentBidder = memberOff[1]["name"]
				minBid = minBidMOS
				SendChatMessage("Moving to member off specs! "..currentBidder.." min "..minBid, channel)
				SendChatMessage("It is your bid! Min "..minBid..".", "WHISPER", nil, currentBidder)
			elseif (trialMain[1]) then
				bidCategory = "tm"
				bidderIndex = 1
				currentBidder = trialMain[1]["name"]
				minBid = minBidTMS
				SendChatMessage("Moving to trial main specs! "..currentBidder.." min "..minBid, channel)
				SendChatMessage("It is your bid! Min "..minBid..".", "WHISPER", nil, currentBidder)
			elseif (trialOff[1]) then
				bidCategory = "to"
				bidderIndex = 1
				currentBidder = trialOff[1]["name"]
				minBid = minBidTOS
				SendChatMessage("Moving to trial off specs! "..currentBidder.." min "..minBid, channel)
				SendChatMessage("It is your bid! Min "..minBid..".", "WHISPER", nil, currentBidder)
			else
				if (bMult) then SendChatMessage(item.." #"..curNum..": DE", channel)
				else SendChatMessage(item..": DE", channel) end
				local name, _, _, ilvl = GetItemInfo(item)
				local hIndicator = ARP_GetLootIndicator(ilvl)
				SendAddonMessage("ARP_LOG", name..hIndicator.." = DE", "RAID")	
				if (itemCount > 1) then
					itemCount = itemCount - 1
					curNum = curNum + 1
					currentBidder = nil
					topBidder = nil
					ARP_StartBid()
				else
					ARP_Reset()
				end
			end
		elseif (bidCategory == "mo") then
			if (trialMain[1]) then
				bidCategory = "tm"
				bidderIndex = 1
				currentBidder = trialMain[1]["name"]
				minBid = minBidTMS
				SendChatMessage("Moving to trial main specs! "..currentBidder.." min "..minBid, channel)
				SendChatMessage("It is your bid! Min "..minBid..".", "WHISPER", nil, currentBidder)
			elseif (trialOff[1]) then
				bidCategory = "to"
				bidderIndex = 1
				currentBidder = trialOff[1]["name"]
				minBid = minBidTOS
				SendChatMessage("Moving to trial off specs! "..currentBidder.." min "..minBid, channel)
				SendChatMessage("It is your bid! Min "..minBid..".", "WHISPER", nil, currentBidder)
			else
				if (bMult) then SendChatMessage(item.." #"..curNum..": DE", channel)
				else SendChatMessage(item..": DE", channel) end
				local name, _, _, ilvl = GetItemInfo(item)
				local hIndicator = ARP_GetLootIndicator(ilvl)
				SendAddonMessage("ARP_LOG", name..hIndicator.." = DE", "RAID")	
				if (itemCount > 1) then
					itemCount = itemCount - 1
					curNum = curNum + 1
					currentBidder = nil
					topBidder = nil
					ARP_StartBid()
				else
					ARP_Reset()
				end
			end
		elseif (bidCategory == "tm") then
			if (trialOff[1]) then
			bidCategory = "to"
				bidderIndex = 1
				currentBidder = trialOff[1]["name"]
				minBid = minBidTOS
				SendChatMessage("Moving to trial off specs! "..currentBidder.." min "..minBid, channel)
				SendChatMessage("It is your bid! Min "..minBid..".", "WHISPER", nil, currentBidder)
			else
				if (bMult) then SendChatMessage(item.." #"..curNum..": DE", channel)
				else SendChatMessage(item..": DE", channel) end
				local name, _, _, ilvl = GetItemInfo(item)
				local hIndicator = ARP_GetLootIndicator(ilvl)
				SendAddonMessage("ARP_LOG", name..hIndicator.." = DE", "RAID")	
				if (itemCount > 1) then
					itemCount = itemCount - 1
					curNum = curNum + 1
					currentBidder = nil
					topBidder = nil
					ARP_StartBid()
				else
					ARP_Reset()
				end
			end
		else
			if (bMult) then SendChatMessage(item.." #"..curNum..": DE", channel)
			else SendChatMessage(item..": DE", channel) end
			local name, _, _, ilvl = GetItemInfo(item)
			local hIndicator = ARP_GetLootIndicator(ilvl)
			SendAddonMessage("ARP_LOG", name..hIndicator.." = DE", "RAID")	
			if (itemCount > 1) then
				itemCount = itemCount - 1
				curNum = curNum + 1
				currentBidder = nil
				topBidder = nil
				ARP_StartBid()
			else
				ARP_Reset()
			end
		end		
	else
		if (bidCategory == "mm" or bidCategory == "tm") then
			if (winAmt < 10) then winAmt = 10 end
		else
			if (winAmt < 1) then winAmt = 1 end
		end
		if (bMult) then SendChatMessage(item.." #"..curNum..": "..winner.." for "..winAmt, channel)
		else SendChatMessage(item..": "..winner.." for "..winAmt, channel) end
		local name, _, _, ilvl = GetItemInfo(item)
		local hIndicator = ARP_GetLootIndicator(ilvl)
		SendAddonMessage("ARP_LOG", name..hIndicator.." = "..winner.." "..winAmt, "RAID")	
		if (itemCount > 1) then
			itemCount = itemCount - 1
			curNum = curNum + 1
			currentBidder = nil
			topBidder = nil
			minBid = nil
			ARP_RemoveWinner(winner)
			ARP_StartBid()
		else 
			ARP_Reset() 
		end
	end
end

function ARP_GetLootIndicator(ilvl)
	for i,v in pairs(heroicLoot) do
		if (ilvl == v) then return " (H)" end
	end
	for i,v in pairs(ilvlLoot) do
		if (ilvl == v) then return " ("..ilvl..")" end
	end
	return ""
end

function ARP_RemoveWinner(winner)
	for i,v in pairs(ARP_GetList()) do
		if (v["name"] == winner) then
			table.remove(ARP_GetList(), i)
			break
		end
	end
end

function ARP_StartBid()
	if (not item) then print ("You have not been assigned an item to bid!")
	else
		ARP_BuildBidString(item)
		if (not memberMain[1] and not memberOff[1] and not trialMain[1] and not trialOff[1]) then
			if (bMult) then 
				if (itemCount == 1) then 
					SendChatMessage(item.." #"..curNum..": DE", channel)
					local name, _, _, ilvl = GetItemInfo(item)
					local hIndicator = ARP_GetLootIndicator(ilvl)
					SendAddonMessage("ARP_LOG", name..hIndicator.." = DE", "RAID")	
					ARP_Reset() 
					return
				end
				SendChatMessage(item.." #"..curNum..": DE", channel)
				itemCount = itemCount - 1
				curNum = curNum + 1
				local name, _, _, ilvl = GetItemInfo(item)
				local hIndicator = ARP_GetLootIndicator(ilvl)
				SendAddonMessage("ARP_LOG", name..hIndicator.." = DE", "RAID")	
				ARP_StartBid()
			else 
				SendChatMessage(item..": DE", channel) 
				local name, _, _, ilvl = GetItemInfo(item)
				local hIndicator = ARP_GetLootIndicator(ilvl)
				SendAddonMessage("ARP_LOG", name..hIndicator.." = DE", "RAID")	
				ARP_Reset()
			end
		else
			ARP_BidItem(channel)
		end
	end
end

function ARP_UpdateBid(bidAmt)
	local list = ARP_GetList(bidCategory)
	local msg = ""
	if (bidAmt == -1) then
		if (bidderIndex == #list) then 
			SendChatMessage(item..": "..currentBidder.." has bid "..minBid.."! No more bidders.", channel)
			ARP_EndBid(currentBidder, minBid)
		else
			bidderIndex = bidderIndex + 1
			for i=bidderIndex, #list do
				if (i == #list) then msg = (msg..list[i]["name"])
				else msg = (msg..list[i]["name"]..", ") end
			end
			msg = (item..": "..currentBidder.." has bid "..minBid.."! Remaining bidders: "..msg)
			SendChatMessage(msg, channel)
			topBidder = currentBidder
			currentBidder = list[bidderIndex]["name"]
		end
	elseif (bidAmt == "pass") then
		if (bidderIndex == #list) then
			SendChatMessage(item..": "..currentBidder.." has passed! No more bidders.", channel)
			ARP_EndBid(topBidder, minBid-1)
		else
			bidderIndex = bidderIndex + 1			
			for i=bidderIndex, #list do
				if (i == #list) then msg = (msg..list[i]["name"])
				else msg = (msg..list[i]["name"]..", ") end
			end
			if (topBidder == nil) then topBidder = "None" end
			msg = (item..": "..currentBidder.." has passed! Top bidder: "..topBidder.."; Remaining bidders: "..msg)
			SendChatMessage(msg, channel)
			currentBidder = list[bidderIndex]["name"]
			SendChatMessage("It is your bid! Min "..minBid..".", "WHISPER", nil, currentBidder)
		end
	else
		if (string.sub(bidAmt, 1, 1) == "+") then bidAmt = ((minBid-1) + tonumber(bidAmt))
		elseif (bidAmt == "yup") then bidAmt = minBid
		else bidAmt = tonumber(bidAmt) end
		if (bidAmt < minBid) then
			SendChatMessage("Your bid is too low! The minimum is "..minBid..".", "WHISPER", nil, currentBidder)
		elseif (bidAmt > list[bidderIndex]["points"]) then
			SendChatMessage("You don't have that many points! You have "..list[bidderIndex]["points"]..".", "WHISPER", nil, currentBidder)
		else
			topBidder = currentBidder
			if (bidderIndex == #list) then
				if (bidAmt > minBid) then bidAmt = minBid end
				SendChatMessage(item..": "..currentBidder.." has bid "..bidAmt..". No more bidders!", channel)
				ARP_EndBid(topBidder, bidAmt)
			else
				minBid = bidAmt + 1
				bidderIndex = bidderIndex + 1
				for i=bidderIndex, #list do
					if (i == #list) then msg = (msg..list[i]["name"])
					else msg = (msg..list[i]["name"]..", ") end
				end
				msg = (item..": "..currentBidder.." has bid "..bidAmt..". Remaining bidders: "..msg)
				SendChatMessage(msg, channel)
				currentBidder = list[bidderIndex]["name"]
				SendChatMessage("It is your bid! Min "..minBid..".", "WHISPER", nil, currentBidder)
			end
		end
	end
end

function ARP_UpdateList(list, name, points, other_list)
	local replacing = false
	local found = nil
	local randnum = nil
	
	for i,v in ipairs(other_list) do
		if (v["name"] == name) then
			randnum = v["randnum"]
			found = i
		end
	end
	if (found ~= nil) then table.remove(other_list, found) end
		
	for i,v in ipairs(list) do
		if (v["name"] == name) then
			v["points"] = points
			replacing = true
			break
		end
	end
	
	if (not replacing) then 
		if (randnum == nil) then randnum = math.random() end
		table.insert(list, {ID = #list+1, name = name, points = points, randnum = randnum})
	end	
end

--------------------)
---Misc Functions---)
--------------------)
function ARP_GetList()
	if (bidCategory == "mm") then return memberMain
	elseif (bidCategory == "mo") then return memberOff
	elseif (bidCategory == "tm") then return trialMain
	elseif (bidCategory == "to") then return trialOff
	end
end

function ARP_Reset()
	memberMain = {}
	memberOff = {}
	trialMain = {}
	trialOff = {}
	
	currentBidder = nil
	bidderIndex = nil
	topBidder = nil
	item = nil
	itemCount = 1
	curNum = 1
	
	bBidding = false
	bMult = false
	SendAddonMessage("ARP_READY", nil, "RAID")
	ARP_MLPanel_Update()
end

function ARP_Sort(list)
	table.sort(list, function(a, b)
		if a["points"] == b["points"] then 
			return a["randnum"] < b["randnum"] 
		else 	
			return a["points"] < b["points"]
		end
	end)
	return list
end
